@@ -23,58 +23,47 @@ from nise.generators.aws.aws_generator import AWSGenerator
 class S3Generator(AWSGenerator):
     """Generator for S3 data."""
 
-    def __init__(self, start_date, end_date, payer_account, usage_accounts, attributes=None, tag_cols=None):
-        """Initialize the S3 generator."""
-        super().__init__(start_date, end_date, payer_account, usage_accounts, attributes, tag_cols)
-        self._amount = uniform(0.2, 6000.99)
-        self._rate = round(uniform(0.02, 0.06), 3)
-        self._product_sku = self.fake.pystr(min_chars=12, max_chars=12).upper()
-        self._resource_id = self.fake.ean8()
-        if self.attributes:
-            if self.attributes.get("amount"):
-                self._amount = float(self.attributes.get("amount"))
-            if self.attributes.get("rate"):
-                self._rate = float(self.attributes.get("rate"))
-            if self.attributes.get("product_sku"):
-                self._product_sku = self.attributes.get("product_sku")
-            if self.attributes.get("resource_id"):
-                self._resource_id = self.attributes.get("resource_id")
-            if self.attributes.get("tags"):
-                self._tags = self.attributes.get("tags")
+    def _gen_fake_data(self, count):
+        """Populate TEMPLATE_KWARGS with fake values."""
+        self.TEMPLATE_KWARGS["s3_gens"] = []
+        while len(self.TEMPLATE_KWARGS["s3_gens"]) < count:
+            self.TEMPLATE_KWARGS["s3_gens"].append(
+                {"amount": uniform(0.2, 6000.99), "rate": round(uniform(0.02, 0.06), 3)}
+            )
 
-    def _get_arn(self, avail_zone):
+    def _get_arn(self, avail_zone, config):
         """Create an amazon resource name."""
-        return f"arn:aws:ec2:{avail_zone}:{self.payer_account}:snapshot/snap-{self._resource_id}"
+        return f"arn:aws:ec2:{avail_zone}:{config.get('payer_account')}:snapshot/snap-{config.get('resource_id')}"
 
     def _update_data(self, row, start, end, **kwargs):
         """Update data with generator specific data."""
+        current_config = kwargs.get("config", {})
+
         row = self._add_common_usage_info(row, start, end)
 
-        rate = self._rate
-        amount = self._amount
+        rate = current_config.get("rate")
+        amount = current_config.get("amount")
         cost = amount * rate
-        location, aws_region, avail_zone, _ = self._get_location()
-        description = f"${rate} per GB-Month of snapshot data stored - {location}"
-        amazon_resource_name = self._get_arn(avail_zone)
+        location, aws_region, avail_zone, _ = self._get_location(config=current_config)
 
         row["lineItem/ProductCode"] = "AmazonS3"
         row["lineItem/UsageType"] = "Requests-Tier2"
         row["lineItem/Operation"] = "GetObject"
-        row["lineItem/ResourceId"] = amazon_resource_name
+        row["lineItem/ResourceId"] = self._get_arn(avail_zone, config=current_config)
         row["lineItem/UsageAmount"] = str(amount)
         row["lineItem/CurrencyCode"] = "USD"
         row["lineItem/UnblendedRate"] = str(rate)
         row["lineItem/UnblendedCost"] = str(cost)
         row["lineItem/BlendedRate"] = str(rate)
         row["lineItem/BlendedCost"] = str(cost)
-        row["lineItem/LineItemDescription"] = description
+        row["lineItem/LineItemDescription"] = f"${rate} per GB-Month of snapshot data stored - {location}"
         row["product/ProductName"] = "Amazon Simple Storage Service"
         row["product/location"] = location
         row["product/locationType"] = "AWS Region"
         row["product/productFamily"] = "Storage Snapshot"
         row["product/region"] = aws_region
         row["product/servicecode"] = "AmazonS3"
-        row["product/sku"] = self._product_sku
+        row["product/sku"] = current_config.get("product_sku")
         row["product/storageMedia"] = "Amazon S3"
         row["product/usagetype"] = "Requests-Tier2"
         row["pricing/publicOnDemandCost"] = str(cost)
