@@ -18,7 +18,6 @@
 from nise.generators.azure.azure_generator import AzureGenerator
 
 
-# pylint: disable=too-many-arguments, too-many-instance-attributes
 class SQLGenerator(AzureGenerator):
     """Generator for SQL Database data."""
 
@@ -35,8 +34,17 @@ class SQLGenerator(AzureGenerator):
         ("hccm", "costmgmtacct1234"),
     )
     ADDITIONAL_INFO = ({"ConsumptionMeter": "22222222-3333-4444-5555-666666666666"},)
+    SERVICE_NAME = "SQL Database"
 
-    def __init__(self, start_date, end_date, payer_account, usage_accounts, attributes=None):
-        """Initialize the data transfer generator."""
-        self._service_name = "SQL Database"
-        super().__init__(start_date, end_date, payer_account, usage_accounts, attributes)
+    def _gen_fake_data(self, count):
+        """Populate TEMPLATE_KWARGS with fake values.
+
+        The base template has defaults for most values. The values set here have extra requirements.
+        """
+        self.TEMPLATE_KWARGS["sql_gens"] = []
+        while len(self.TEMPLATE_KWARGS["sql_gens"]) < count:
+            # this might look strange, but because the Azure template has defaults for each element, we don't need to
+            # populate the dict with anything. The list just needs to have the right number of elements.
+            #
+            # If a future change needs to push default values into the template, the values may be added here.
+            self.TEMPLATE_KWARGS["sql_gens"].append({})
