@@ -20,7 +20,6 @@ import calendar
 import copy
 import csv
 import gzip
-import importlib
 import json
 import os
 import random
@@ -28,33 +27,34 @@ import shutil
 import string
 import tarfile
 from datetime import datetime
-from random import randint
 from tempfile import gettempdir
 from tempfile import NamedTemporaryFile
 from tempfile import TemporaryDirectory
 from uuid import uuid4
 
 import requests
-from dateutil import parser
 from dateutil.relativedelta import relativedelta
 from faker import Faker
 from nise.copy import copy_to_local_dir
 from nise.extract import extract_payload
-from nise.generators.aws import DataTransferGenerator
-from nise.generators.aws import EBSGenerator
-from nise.generators.aws import EC2Generator
-from nise.generators.aws import RDSGenerator
-from nise.generators.aws import Route53Generator
-from nise.generators.aws import S3Generator
-from nise.generators.aws import VPCGenerator
+from nise.generators.aws import AWS_GENERATORS
+from nise.generators.aws import DataTransferGenerator  # noqa: F401
+from nise.generators.aws import EBSGenerator  # noqa: F401
+from nise.generators.aws import EC2Generator  # noqa: F401
+from nise.generators.aws import RDSGenerator  # noqa: F401
+from nise.generators.aws import Route53Generator  # noqa: F401
+from nise.generators.aws import S3Generator  # noqa: F401
+from nise.generators.aws import VPCGenerator  # noqa: F401
 from nise.generators.azure import AZURE_COLUMNS
-from nise.generators.azure import BandwidthGenerator
-from nise.generators.azure import SQLGenerator
-from nise.generators.azure import StorageGenerator
-from nise.generators.azure import VMGenerator
-from nise.generators.azure import VNGenerator
-from nise.generators.gcp import CloudStorageGenerator
-from nise.generators.gcp import ComputeEngineGenerator
+from nise.generators.azure import AZURE_GENERATORS
+from nise.generators.azure import BandwidthGenerator  # noqa: F401
+from nise.generators.azure import SQLGenerator  # noqa: F401
+from nise.generators.azure import StorageGenerator  # noqa: F401
+from nise.generators.azure import VMGenerator  # noqa: F401
+from nise.generators.azure import VNGenerator  # noqa: F401
+from nise.generators.gcp import CloudStorageGenerator  # noqa: F401
+from nise.generators.gcp import ComputeEngineGenerator  # noqa: F401
+from nise.generators.gcp import GCP_GENERATORS
 from nise.generators.gcp import GCP_REPORT_COLUMNS
 from nise.generators.gcp import ProjectGenerator
 from nise.generators.ocp import OCP_NODE_LABEL
@@ -67,6 +67,7 @@ from nise.manifest import ocp_generate_manifest
 from nise.upload import upload_to_azure_container
 from nise.upload import upload_to_gcp_storage
 from nise.upload import upload_to_s3
+from nise.util import load_yaml
 from nise.util import LOG
 
 
@@ -269,56 +270,15 @@ def _create_month_list(start_date, end_date):
     return months
 
 
-def _aws_finalize_report(data, static_data=None):
+def _aws_finalize_report(data):
     """Populate invoice id for data."""
     data = copy.deepcopy(data)
-
-    invoice_id = None
-    if static_data and static_data.get("finalized_report"):
-        invoice_id = static_data.get("finalized_report").get("invoice_id")
-
-    if not invoice_id:
-        invoice_id = "".join([random.choice(string.digits) for _ in range(9)])
+    invoice_id = "".join([random.choice(string.digits) for _ in range(9)])
     for row in data:
         row["bill/InvoiceId"] = invoice_id
-
     return data
 
 
-def _generate_accounts(static_report_data=None):
-    """Generate payer and usage accounts."""
-    if static_report_data:
-        payer_account = static_report_data.get("payer")
-        usage_accounts = tuple(static_report_data.get("user"))
-    else:
-        fake = Faker()
-        payer_account = fake.ean(length=13)
-        usage_accounts = (
-            payer_account,
-            fake.ean(length=13),
-            fake.ean(length=13),
-            fake.ean(length=13),
-            fake.ean(length=13),
-        )
-    return payer_account, usage_accounts
-
-
-def _get_generators(generator_list):
-    """Collect a list of report generators."""
-    generators = []
-    if generator_list:
-        for item in generator_list:
-            for generator_cls, attributes in item.items():
-                generator_obj = {"generator": getattr(importlib.import_module(__name__), generator_cls)}
-                if attributes.get("start_date"):
-                    attributes["start_date"] = parser.parse(attributes.get("start_date"))
-                if attributes.get("end_date"):
-                    attributes["end_date"] = parser.parse(attributes.get("end_date"))
-                generator_obj["attributes"] = attributes
-                generators.append(generator_obj)
-    return generators
-
-
 def _create_generator_dates_from_yaml(attributes, month):
     """Calculate generator start and end dates based on yaml and current month."""
     gen_start_date = None
@@ -355,9 +315,7 @@ def _create_generator_dates_from_yaml(attributes, month):
     return gen_start_date, gen_end_date
 
 
-def write_aws_file(
-    file_number, aws_report_name, month_name, year, data, aws_finalize_report, static_report_data, headers
-):
+def write_aws_file(file_number, aws_report_name, month_name, year, data, aws_finalize_report, headers):
     """Write AWS data to a file."""
     headers = sorted(list(headers))
     if file_number != 0:
@@ -366,10 +324,10 @@ def write_aws_file(
         file_name = f"{month_name}-{year}-{aws_report_name}"
 
     if aws_finalize_report and aws_finalize_report == "overwrite":
-        data = _aws_finalize_report(data, static_report_data)
+        data = _aws_finalize_report(data)
     elif aws_finalize_report and aws_finalize_report == "copy":
         # Currently only a local option as this does not simulate
-        finalized_data = _aws_finalize_report(data, static_report_data)
+        finalized_data = _aws_finalize_report(data)
         file_name_finalized = f"{file_name}-finalized"
         full_file_name = "{}/{}.csv".format(os.getcwd(), file_name_finalized)
         _write_csv(full_file_name, finalized_data, headers)
@@ -386,57 +344,34 @@ def aws_create_report(options):  # noqa: C901
     start_date = options.get("start_date")
     end_date = options.get("end_date")
     aws_finalize_report = options.get("aws_finalize_report")
-    static_report_data = options.get("static_report_data")
-
-    if static_report_data:
-        generators = _get_generators(static_report_data.get("generators"))
-        accounts_list = static_report_data.get("accounts")
-    else:
-        generators = [
-            {"generator": DataTransferGenerator, "attributes": None},
-            {"generator": EBSGenerator, "attributes": None},
-            {"generator": EC2Generator, "attributes": None},
-            {"generator": S3Generator, "attributes": None},
-            {"generator": RDSGenerator, "attributes": None},
-            {"generator": Route53Generator, "attributes": None},
-            {"generator": VPCGenerator, "attributes": None},
-        ]
-        accounts_list = None
 
     months = _create_month_list(start_date, end_date)
 
-    payer_account, usage_accounts = _generate_accounts(accounts_list)
-
     aws_bucket_name = options.get("aws_bucket_name")
     aws_report_name = options.get("aws_report_name")
     write_monthly = options.get("write_monthly", False)
+    payer_account = None
     for month in months:
         data = []
         file_number = 0
         monthly_files = []
-        fake = Faker()
-        num_gens = len(generators)
+        num_gens = len(AWS_GENERATORS)
         ten_percent = int(num_gens * 0.1) if num_gens > 50 else 5
         LOG.info(f"Producing data for {num_gens} generators for {month.get('start').strftime('%Y-%m')}.")
-        for count, generator in enumerate(generators):
-            generator_cls = generator.get("generator")
-            attributes = generator.get("attributes")
+        for count, generator in enumerate(AWS_GENERATORS):
             gen_start_date = month.get("start")
             gen_end_date = month.get("end")
-            if attributes:
-                # Skip if generator usage is outside of current month
-                if attributes.get("end_date") < month.get("start"):
-                    continue
-                if attributes.get("start_date") > month.get("end"):
-                    continue
+            # Skip if generator usage is outside of current month
+            if end_date < month.get("start"):
+                continue
+            if start_date > month.get("end"):
+                continue
 
-                gen_start_date, gen_end_date = _create_generator_dates_from_yaml(attributes, month)
+            gen_start_date, gen_end_date = _create_generator_dates_from_yaml(options, month)
 
-            gen = generator_cls(
-                gen_start_date, gen_end_date, payer_account, usage_accounts, attributes, options.get("aws_tags")
-            )
-            num_instances = 1 if attributes else randint(2, 60)
-            for _ in range(num_instances):
+            gen = generator(gen_start_date, gen_end_date, user_config=options.get("static_report_file"))
+            payer_account = gen.config[0].get("accounts", {}).get("payer")
+            for _ in range(gen.num_instances):
                 for hour in gen.generate_data():
                     data += [hour]
                     if len(data) == options.get("row_limit"):
@@ -448,7 +383,6 @@ def aws_create_report(options):  # noqa: C901
                             gen_start_date.year,
                             data,
                             aws_finalize_report,
-                            static_report_data,
                             gen.AWS_COLUMNS,
                         )
                         monthly_files.append(month_output_file)
@@ -459,6 +393,7 @@ def aws_create_report(options):  # noqa: C901
 
         if file_number != 0:
             file_number += 1
+
         month_output_file = write_aws_file(
             file_number,
             aws_report_name,
@@ -466,7 +401,6 @@ def aws_create_report(options):  # noqa: C901
             gen_start_date.year,
             data,
             aws_finalize_report,
-            static_report_data,
             gen.AWS_COLUMNS,
         )
         monthly_files.append(month_output_file)
@@ -477,7 +411,7 @@ def aws_create_report(options):  # noqa: C901
             manifest_values["start_date"] = gen_start_date
             manifest_values["end_date"] = gen_end_date
             manifest_values["file_names"] = monthly_files
-            s3_cur_path, manifest_data = aws_generate_manifest(fake, manifest_values)
+            s3_cur_path, manifest_data = aws_generate_manifest(manifest_values)
             s3_month_path = os.path.dirname(s3_cur_path)
             s3_month_manifest_path = s3_month_path + "/" + aws_report_name + "-Manifest.json"
             s3_assembly_manifest_path = s3_cur_path + "/" + aws_report_name + "-Manifest.json"
@@ -501,24 +435,9 @@ def azure_create_report(options):  # noqa: C901
     data = []
     start_date = options.get("start_date")
     end_date = options.get("end_date")
-    static_report_data = options.get("static_report_data")
-    if static_report_data:
-        generators = _get_generators(static_report_data.get("generators"))
-        accounts_list = static_report_data.get("accounts")
-    else:
-        generators = [
-            {"generator": BandwidthGenerator, "attributes": {}},
-            {"generator": SQLGenerator, "attributes": {}},
-            {"generator": StorageGenerator, "attributes": {}},
-            {"generator": VMGenerator, "attributes": {}},
-            {"generator": VNGenerator, "attributes": {}},
-        ]
-        accounts_list = None
 
     months = _create_month_list(start_date, end_date)
 
-    payer_account, usage_accounts = _generate_accounts(accounts_list)
-
     meter_cache = {}
     # The options params are not going to change so we don't
     # have to keep resetting the var inside of the for loop
@@ -530,27 +449,21 @@ def azure_create_report(options):  # noqa: C901
     for month in months:
         data = []
         monthly_files = []
-        num_gens = len(generators)
+        num_gens = len(AZURE_GENERATORS)
         ten_percent = int(num_gens * 0.1) if num_gens > 50 else 5
         LOG.info(f"Producing data for {num_gens} generators for {month.get('start').strftime('%Y-%m')}.")
-        for count, generator in enumerate(generators):
-            generator_cls = generator.get("generator")
-            attributes = generator.get("attributes", {})
+        for count, generator in enumerate(AZURE_GENERATORS):
             gen_start_date = month.get("start")
             gen_end_date = month.get("end")
-            if attributes:
-                # Skip if generator usage is outside of current month
-                if attributes.get("end_date") < month.get("start"):
-                    continue
-                if attributes.get("start_date") > month.get("end"):
-                    continue
-            else:
-                attributes = {"end_date": end_date, "start_date": start_date}
+            # Skip if generator usage is outside of current month
+            if end_date < month.get("start"):
+                continue
+            if start_date > month.get("end"):
+                continue
 
-            gen_start_date, gen_end_date = _create_generator_dates_from_yaml(attributes, month)
+            gen_start_date, gen_end_date = _create_generator_dates_from_yaml(options, month)
 
-            attributes["meter_cache"] = meter_cache
-            gen = generator_cls(gen_start_date, gen_end_date, payer_account, usage_accounts, attributes)
+            gen = generator(gen_start_date, gen_end_date, meter_cache, user_config=options.get("static_report_file"))
             data += gen.generate_data()
             meter_cache = gen.get_meter_cache()
 
@@ -597,53 +510,46 @@ def write_ocp_file(file_number, cluster_id, month_name, year, report_type, data)
 
 def ocp_create_report(options):  # noqa: C901
     """Create a usage report file."""
+    cluster_id = options.get("ocp_cluster_id")
     start_date = options.get("start_date")
     end_date = options.get("end_date")
-    cluster_id = options.get("ocp_cluster_id")
-    static_report_data = options.get("static_report_data")
-    if static_report_data:
-        generators = _get_generators(static_report_data.get("generators"))
-    else:
-        generators = [{"generator": OCPGenerator, "attributes": None}]
+    months = _create_month_list(start_date, end_date)  # FIXME: DRY this up
 
-    months = _create_month_list(start_date, end_date)
-    insights_upload = options.get("insights_upload")
     write_monthly = options.get("write_monthly", False)
     for month in months:
         data = {OCP_POD_USAGE: [], OCP_STORAGE_USAGE: [], OCP_NODE_LABEL: []}
         file_numbers = {OCP_POD_USAGE: 0, OCP_STORAGE_USAGE: 0, OCP_NODE_LABEL: 0}
         monthly_files = []
-        for generator in generators:
-            generator_cls = generator.get("generator")
-            attributes = generator.get("attributes")
-            gen_start_date = month.get("start")
-            gen_end_date = month.get("end")
-            if attributes:
-                # Skip if generator usage is outside of current month
-                if attributes.get("end_date") < month.get("start"):
-                    continue
-                if attributes.get("start_date") > month.get("end"):
-                    continue
 
-                gen_start_date, gen_end_date = _create_generator_dates_from_yaml(attributes, month)
+        gen_start_date = month.get("start")
+        gen_end_date = month.get("end")
 
-            gen = generator_cls(gen_start_date, gen_end_date, attributes)
-            for report_type in gen.ocp_report_generation.keys():
-                LOG.info(f"Generating data for {report_type} for {month.get('name')}")
-                for hour in gen.generate_data(report_type):
-                    data[report_type] += [hour]
-                    if len(data[report_type]) == options.get("row_limit"):
-                        file_numbers[report_type] += 1
-                        month_output_file = write_ocp_file(
-                            file_numbers[report_type],
-                            cluster_id,
-                            month.get("name"),
-                            gen_start_date.year,
-                            report_type,
-                            data[report_type],
-                        )
-                        monthly_files.append(month_output_file)
-                        data[report_type].clear()
+        # Skip if generator usage is outside of current month
+        if end_date < month.get("start"):
+            continue
+        if start_date > month.get("end"):
+            continue
+
+        gen_start_date, gen_end_date = _create_generator_dates_from_yaml(options, month)
+
+        gen = OCPGenerator(gen_start_date, gen_end_date, user_config=options.get("static_report_file"))
+        for report_type in gen.ocp_report_generation.keys():
+            LOG.info(f"Generating data for {report_type} for {month.get('name')}")
+            for hour in gen.generate_data(report_type):
+                data[report_type] += [hour]
+
+                if len(data[report_type]) == options.get("row_limit"):
+                    file_numbers[report_type] += 1
+                    month_output_file = write_ocp_file(
+                        file_numbers[report_type],
+                        cluster_id,
+                        month.get("name"),
+                        gen_start_date.year,
+                        report_type,
+                        data[report_type],
+                    )
+                    monthly_files.append(month_output_file)
+                    data[report_type].clear()
 
         for report_type in gen.ocp_report_generation.keys():
             if file_numbers[report_type] != 0:
@@ -659,6 +565,7 @@ def ocp_create_report(options):  # noqa: C901
             )
             monthly_files.append(month_output_file)
 
+        insights_upload = options.get("insights_upload")
         if insights_upload:
             # Generate manifest for all files
             ocp_assembly_id = uuid4()
@@ -698,41 +605,32 @@ def ocp_create_report(options):  # noqa: C901
 def gcp_create_report(options):  # noqa: C901
     """Create a GCP cost usage report file."""
     fake = Faker()
-
     report_prefix = options.get("gcp_report_prefix") or fake.word()
     gcp_bucket_name = options.get("gcp_bucket_name")
 
     start_date = options.get("start_date")
     end_date = options.get("end_date")
 
-    static_report_data = options.get("static_report_data")
-    if static_report_data:
-        generators = _get_generators(static_report_data.get("generators"))
-        projects = static_report_data.get("projects")
-
+    projects = []
+    if options.get("static_report_file"):
+        config = load_yaml(options.get("static_report_file"))
+        project_gens = list(filter(lambda x: "ProjectGenerator" in x, config.get("generators")))
+        projects = []
+        for gen in project_gens:
+            project_generator = ProjectGenerator(gen.get("ProjectGenerator", {}).get("Account ID"))
+            projects = projects + [prj for prj in project_generator.generate_projects()]
     else:
-        generators = [
-            {"generator": CloudStorageGenerator, "attributes": None},
-            {"generator": ComputeEngineGenerator, "attributes": None},
-        ]
         account = "{}-{}".format(fake.word(), fake.word())
-
         project_generator = ProjectGenerator(account)
-        projects = project_generator.generate_projects()
+        projects = projects + [prj for prj in project_generator.generate_projects()]
 
     data = {}
     for project in projects:
-        num_gens = len(generators)
+        num_gens = len(GCP_GENERATORS)
         ten_percent = int(num_gens * 0.1) if num_gens > 50 else 5
-        LOG.info(f"Producing data for {num_gens} generators for {'INSERT SOMETHING FOR GCP'}.")
-        for count, generator in enumerate(generators):
-            attributes = generator.get("attributes", {})
-            if attributes:
-                start_date = attributes.get("start_date")
-                end_date = attributes.get("end_date")
-
-            generator_cls = generator.get("generator")
-            gen = generator_cls(start_date, end_date, project, attributes=attributes)
+        LOG.info(f"Producing data for {num_gens} generators for GCP Project '{project}'.")
+        for count, generator in enumerate(GCP_GENERATORS):
+            gen = generator(start_date, end_date, project, user_config=options.get("static_report_file"))
             generated_data = gen.generate_data()
             for key, item in generated_data.items():
                 if key in data:
