@@ -14,13 +14,13 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 """Utility to generate koku-nise OCP yaml files."""
-import logging
 import os
 from calendar import monthrange
 from datetime import date
 
 import faker
 from dateutil.relativedelta import relativedelta
+from nise.util import LOG
 from nise.yaml_generators.generator import Generator
 from nise.yaml_generators.utils import dicta
 from nise.yaml_generators.utils import generate_name
@@ -28,7 +28,7 @@ from nise.yaml_generators.utils import generate_resource_id
 
 
 FAKER = faker.Faker()
-LOG = logging.getLogger(__name__)
+SEEN_LABELS = set()
 
 
 def generate_labels(num_labels):
@@ -41,7 +41,13 @@ def generate_labels(num_labels):
     Returns:
         str
     """
-    return "|".join(f"label_{e[0]}:{e[1]}" for e in zip(FAKER.words(num_labels), FAKER.words(num_labels)))
+    lis = []
+    while len(lis) < num_labels:
+        e = tuple(FAKER.words(2))
+        if e not in SEEN_LABELS:
+            lis.append(f"label_{e[0]}:{e[1]}")
+            SEEN_LABELS.add(e)
+    return "|".join(lis)
 
 
 class OCPGenerator(Generator):
@@ -68,8 +74,9 @@ class OCPGenerator(Generator):
                 cpu_cores: int,     (config.max_node_cpu_cores)
                 memory_gig: int,    (config.max_node_memory_gig)
                 resource_id: str,   (dynamic)
-                namespaces: [     (number of namespaces controlled by config.max_node_namespaces)
-                    {namespace: str,   (dynamic)
+                labels: str           (dynamic)
+                namespaces: [{     (number of namespaces controlled by config.max_node_namespaces)
+                    namespace_name: namespace str,   (dynamic)
                     pods: [           (number of pods controlled by config.max_node_namespace_pods)
                         pod_name: str,        (dynamic)
                         cpu_request: int,     (config.max_node_namespace_pod_cpu_request)
@@ -102,7 +109,10 @@ class OCPGenerator(Generator):
         """
         LOG.info("Data build starting")
 
-        data = dicta(start_date=str(config.start_date), end_date=str(config.end_date), nodes=[])
+        data = dicta(
+            start_date=str(config.start_date), end_date=str(config.end_date), nodes=[], resourceid_labels=None
+        )
+        resourceid_labels = {}
 
         if _random:
             max_nodes = FAKER.random_int(1, config.max_nodes)
@@ -118,12 +128,18 @@ class OCPGenerator(Generator):
                 cores = config.max_node_cpu_cores
                 memory = config.max_node_memory_gig
 
+            resource_id = generate_resource_id(config)
+            node_name = generate_name(config)
+            id_label_key = (resource_id, node_name)
+            resourceid_labels[id_label_key] = []
+            node_labels = generate_labels(config.max_node_namespace_pod_labels)
             node = dicta(
-                name=generate_name(config),
+                name=node_name,
                 cpu_cores=cores,
                 memory_gig=memory,
-                resource_id=generate_resource_id(config),
+                resource_id=resource_id,
                 namespaces=[],
+                labels=node_labels,
             )
             data.nodes.append(node)
 
@@ -135,7 +151,7 @@ class OCPGenerator(Generator):
             for namespace_ix in range(max_namespaces):
                 LOG.info(f"Building node {node_ix + 1}/{max_nodes}; namespace {namespace_ix + 1}/{max_namespaces}...")
 
-                namespace = dicta(name=generate_name(config, prefix=node.name), pods=[], volumes=[])
+                namespace = dicta(namespace_name=generate_name(config, prefix=node.name), pods=[], volumes=[])
                 node.namespaces.append(namespace)
 
                 if _random:
@@ -160,14 +176,18 @@ class OCPGenerator(Generator):
                         mem_lim = mem_req = node.memory_gig
                         pod_sec = config.max_node_namespace_pod_seconds
 
+                    pod_labels = generate_labels(config.max_node_namespace_pod_labels)
+                    resourceid_labels[id_label_key].append(pod_labels)
                     pod = dicta(
-                        name=generate_name(config, prefix=namespace.name + "-pod", suffix=str(pod_ix), dynamic=False),
+                        pod_name=generate_name(
+                            config, prefix=namespace.namespace_name + "-pod", suffix=str(pod_ix), dynamic=False
+                        ),
                         cpu_request=cpu_req,
                         mem_request_gig=mem_req,
                         cpu_limit=cpu_lim,
                         mem_limit_gig=mem_lim,
                         pod_seconds=pod_sec,
-                        labels=generate_labels(config.max_node_namespace_pod_labels),
+                        labels=pod_labels,
                     )
                     namespace.pods.append(pod)
 
@@ -185,13 +205,15 @@ class OCPGenerator(Generator):
                         storage_cls = config.storage_classes[0]
                         vol_req = config.max_node_namespace_volume_request_gig
 
+                    volume_labels = generate_labels(config.max_node_namespace_volume_labels)
+                    resourceid_labels[id_label_key].append(volume_labels)
                     volume = dicta(
-                        name=generate_name(
-                            config, prefix=namespace.name + "-vol", suffix=str(volume_ix), dynamic=False
+                        volume_name=generate_name(
+                            config, prefix=namespace.namespace_name + "-vol", suffix=str(volume_ix), dynamic=False
                         ),
                         storage_class=storage_cls,
                         volume_request_gig=vol_req,
-                        labels=generate_labels(config.max_node_namespace_volume_labels),
+                        labels=volume_labels,
                         volume_claims=[],
                     )
                     namespace.volumes.append(volume)
@@ -209,20 +231,22 @@ class OCPGenerator(Generator):
 
                         pod_name = namespace.pods[
                             -1 if volume_claim_ix >= len(namespace.pods) else volume_claim_ix
-                        ].name
+                        ].pod_name
+                        volume_claim_labels = generate_labels(config.max_node_namespace_volume_volume_claim_labels)
+                        resourceid_labels[id_label_key].append(volume_claim_labels)
                         volume_claim = dicta(
-                            name=generate_name(
+                            volume_claim_name=generate_name(
                                 config,
-                                prefix=namespace.name + "-vol-claim",
+                                prefix=namespace.namespace_name + "-vol-claim",
                                 suffix=str(volume_claim_ix),
                                 dynamic=False,
                             ),
                             pod_name=pod_name,
-                            labels=generate_labels(config.max_node_namespace_volume_volume_claim_labels),
+                            labels=volume_claim_labels,
                             capacity_gig=cap,
                         )
                         volume.volume_claims.append(volume_claim)
-
+        data.resourceid_labels = resourceid_labels
         return data
 
     def default_config(self):
